@@ -54,6 +54,21 @@ class TestServiceModel(unittest.TestCase):
                 'StringShape': {'type': 'string'}
             }
         }
+        self.error_shapes = {
+            'ExceptionOne': {
+                'exception': True,
+                'type': 'structure',
+                'members': {},
+            },
+            'ExceptionTwo': {
+                'exception': True,
+                'type': 'structure',
+                'members': {},
+                'error': {
+                    'code': 'FooCode'
+                }
+            },
+        }
         self.service_model = model.ServiceModel(self.model)
 
     def test_metadata_available(self):
@@ -90,8 +105,8 @@ class TestServiceModel(unittest.TestCase):
         }
         service_name = 'myservice'
         service_model = model.ServiceModel(service_model, service_name)
-        with self.assertRaisesRegexp(model.UndefinedModelAttributeError,
-                                     service_name):
+        with self.assertRaisesRegex(model.UndefinedModelAttributeError,
+                                    service_name):
             service_model.service_id
 
     def test_operation_does_not_exist(self):
@@ -112,6 +127,23 @@ class TestServiceModel(unittest.TestCase):
         self.assertEqual(repr(self.service_model),
                          'ServiceModel(endpoint-prefix)')
 
+    def test_shape_for_error_code(self):
+        self.model['shapes'].update(self.error_shapes)
+        self.service_model = model.ServiceModel(self.model)
+        shape = self.service_model.shape_for_error_code('ExceptionOne')
+        self.assertEqual(shape.name, 'ExceptionOne')
+        shape = self.service_model.shape_for_error_code('FooCode')
+        self.assertEqual(shape.name, 'ExceptionTwo')
+
+    def test_error_shapes(self):
+        self.model['shapes'].update(self.error_shapes)
+        self.service_model = model.ServiceModel(self.model)
+        error_shapes = self.service_model.error_shapes
+        error_shape_names = [shape.name for shape in error_shapes]
+        self.assertEqual(len(error_shape_names), 2)
+        self.assertIn('ExceptionOne', error_shape_names)
+        self.assertIn('ExceptionTwo', error_shape_names)
+
 
 class TestOperationModelFromService(unittest.TestCase):
     def setUp(self):
@@ -299,6 +331,23 @@ class TestOperationModelFromService(unittest.TestCase):
         operation_two = self.service_model.operation_model('OperationName')
         self.assertFalse(operation_two.is_endpoint_discovery_operation)
 
+    def test_endpoint_discovery_required(self):
+        operation = self.model['operations']['OperationName']
+        operation['endpointdiscovery'] = {'required': True}
+        service_model = model.ServiceModel(self.model)
+        self.assertTrue(service_model.endpoint_discovery_required)
+
+    def test_endpoint_discovery_required_false(self):
+        self.model['operations']['OperationName']['endpointdiscovery'] = {}
+        service_model = model.ServiceModel(self.model)
+        self.assertFalse(service_model.endpoint_discovery_required)
+
+    def test_endpoint_discovery_required_no_value(self):
+        operation = self.model['operations']['OperationName']
+        self.assertTrue(operation.get('endpointdiscovery') is None)
+        service_model = model.ServiceModel(self.model)
+        self.assertFalse(service_model.endpoint_discovery_required)
+
     def test_endpoint_discovery_present(self):
         operation = self.model['operations']['OperationName']
         operation['endpointdiscovery'] = {'required': True}
@@ -675,6 +724,27 @@ class TestShapeResolver(unittest.TestCase):
                          ['NewPassword', 'OldPassword'])
         self.assertEqual(shape.members['OldPassword'].name, 'passwordType')
         self.assertEqual(shape.members['OldPassword'].type_name, 'string')
+        self.assertEqual(shape.error_code, None)
+
+    def test_exception_error_code(self):
+        shapes = {
+            'FooException': {
+                'exception': True,
+                'type': 'structure',
+                'members': {}
+            }
+        }
+        # Test without explicit error code
+        resolver = model.ShapeResolver(shapes)
+        shape = resolver.get_shape_by_name('FooException')
+        self.assertTrue(shape.metadata['exception'])
+        self.assertEqual(shape.error_code, 'FooException')
+        # Test with explicit error code
+        shapes['FooException']['error'] = {'code': 'ExceptionCode'}
+        resolver = model.ShapeResolver(shapes)
+        shape = resolver.get_shape_by_name('FooException')
+        self.assertTrue(shape.metadata['exception'])
+        self.assertEqual(shape.error_code, 'ExceptionCode')
 
     def test_shape_metadata(self):
         shapes = {
