@@ -13,7 +13,7 @@
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
 import botocore.config
-from tests import unittest, create_session, temporary_file
+from tests import unittest, create_session, temporary_file, requires_crt
 import os
 import logging
 import tempfile
@@ -23,6 +23,7 @@ import mock
 
 import botocore.session
 import botocore.exceptions
+from botocore import UNSIGNED
 from botocore.model import ServiceModel
 from botocore import client
 from botocore.hooks import HierarchicalEmitter
@@ -59,12 +60,12 @@ class BaseSessionTest(unittest.TestCase):
             'region': config_chain_builder.create_config_chain(
                 instance_name='region',
                 env_var_names='FOO_REGION',
-                config_property_name='foo_region',
+                config_property_names='foo_region',
             ),
             'data_path': config_chain_builder.create_config_chain(
                 instance_name='data_path',
                 env_var_names='FOO_DATA_PATH',
-                config_property_name='data_path',
+                config_property_names='data_path',
             ),
             'config_file': config_chain_builder.create_config_chain(
                 instance_name='config_file',
@@ -77,11 +78,11 @@ class BaseSessionTest(unittest.TestCase):
             'ca_bundle': config_chain_builder.create_config_chain(
                 instance_name='ca_bundle',
                 env_var_names='FOO_AWS_CA_BUNDLE',
-                config_property_name='foo_ca_bundle',
+                config_property_names='foo_ca_bundle',
             ),
             'api_versions': config_chain_builder.create_config_chain(
                 instance_name='api_versions',
-                config_property_name='foo_api_versions',
+                config_property_names='foo_api_versions',
                 default={},
             ),
         }
@@ -407,6 +408,22 @@ class TestSessionUserAgent(BaseSessionTest):
         self.assertTrue(user_agent.endswith('custom-thing/other'))
         self.assertIn('exec-env/FooEnv', user_agent)
 
+    @requires_crt()
+    def test_crt_user_agent_appended(self):
+        user_agent = self.session.user_agent()
+        self.assertIn(' awscrt/', user_agent)
+        self.assertNotIn('awscrt/Unknown', user_agent)
+
+    @requires_crt()
+    def test_crt_and_extra_user_agent(self):
+        user_agent = self.session.user_agent()
+        self.assertIn(' awscrt/', user_agent)
+        self.assertNotIn('custom-thing/other', user_agent)
+        self.session.user_agent_extra = 'custom-thing/other'
+        user_agent_w_extra = self.session.user_agent()
+        self.assertIn(' awscrt/', user_agent)
+        self.assertTrue(user_agent_w_extra.endswith('custom-thing/other'))
+
 
 class TestConfigLoaderObject(BaseSessionTest):
     def test_config_loader_delegation(self):
@@ -499,6 +516,14 @@ class TestCreateClient(BaseSessionTest):
                 aws_secret_access_key='foo',
             )
 
+    def test_cred_provider_not_called_on_unsigned_client(self):
+        cred_provider = mock.Mock()
+        self.session.register_component(
+            'credential_provider', cred_provider)
+        config = botocore.config.Config(signature_version=UNSIGNED)
+        self.session.create_client('sts', 'us-west-2', config=config)
+        self.assertFalse(cred_provider.load_credentials.called)
+
     @mock.patch('botocore.client.ClientCreator')
     def test_config_passed_to_client_creator(self, client_creator):
         # Make sure there is no default set
@@ -711,6 +736,75 @@ class TestSessionComponent(BaseSessionTest):
         )
 
 
+class TestClientMonitoring(BaseSessionTest):
+    def assert_created_client_is_monitored(self, session):
+        with mock.patch('botocore.monitoring.Monitor',
+                        spec=True) as mock_monitor:
+            client = session.create_client('ec2', 'us-west-2')
+        mock_monitor.return_value.register.assert_called_with(
+            client.meta.events)
+
+    def assert_monitoring_host_and_port(self, session, host, port):
+        with mock.patch('botocore.monitoring.SocketPublisher',
+                        spec=True) as mock_publisher:
+            client = session.create_client('ec2', 'us-west-2')
+        self.assertEqual(mock_publisher.call_count, 1)
+        _, args, kwargs = mock_publisher.mock_calls[0]
+        self.assertEqual(kwargs.get('host'), host)
+        self.assertEqual(kwargs.get('port'), port)
+
+    def assert_created_client_is_not_monitored(self, session):
+        with mock.patch('botocore.session.monitoring.Monitor',
+                        spec=True) as mock_monitor:
+            session.create_client('ec2', 'us-west-2')
+            mock_monitor.return_value.register.assert_not_called()
+
+    def test_with_csm_enabled_from_config(self):
+        with temporary_file('w') as f:
+            del self.environ['FOO_PROFILE']
+            self.environ['FOO_CONFIG_FILE'] = f.name
+            f.write('[default]\n')
+            f.write('csm_enabled=true\n')
+            f.flush()
+            self.assert_created_client_is_monitored(self.session)
+
+    def test_with_csm_enabled_from_env(self):
+        self.environ['AWS_CSM_ENABLED'] = 'true'
+        self.assert_created_client_is_monitored(self.session)
+
+    def test_with_csm_host(self):
+        custom_host = '10.13.37.1'
+        self.environ['AWS_CSM_ENABLED'] = 'true'
+        self.environ['AWS_CSM_HOST'] = custom_host
+        self.assert_monitoring_host_and_port(self.session, custom_host, 31000)
+
+    def test_with_csm_port(self):
+        custom_port = '1234'
+        self.environ['AWS_CSM_ENABLED'] = 'true'
+        self.environ['AWS_CSM_PORT'] = custom_port
+        self.assert_monitoring_host_and_port(
+            self.session,
+            '127.0.0.1',
+            int(custom_port),
+        )
+
+    def test_with_csm_disabled_from_config(self):
+        with temporary_file('w') as f:
+            del self.environ['FOO_PROFILE']
+            self.environ['FOO_CONFIG_FILE'] = f.name
+            f.write('[default]\n')
+            f.write('csm_enabled=false\n')
+            f.flush()
+            self.assert_created_client_is_not_monitored(self.session)
+
+    def test_with_csm_disabled_from_env(self):
+        self.environ['AWS_CSM_ENABLED'] = 'false'
+        self.assert_created_client_is_not_monitored(self.session)
+
+    def test_csm_not_configured(self):
+        self.assert_created_client_is_not_monitored(self.session)
+
+
 class TestComponentLocator(unittest.TestCase):
     def setUp(self):
         self.components = botocore.session.ComponentLocator()
@@ -779,3 +873,24 @@ class TestDefaultClientConfig(BaseSessionTest):
         client_config = botocore.config.Config()
         self.session.set_default_client_config(client_config)
         self.assertIs(self.session.get_default_client_config(), client_config)
+
+
+class TestSessionRegionSetup(BaseSessionTest):
+    def test_new_session_with_valid_region(self):
+        s3_client = self.session.create_client('s3', 'us-west-2')
+        self.assertIsInstance(s3_client, client.BaseClient)
+        self.assertEqual(s3_client.meta.region_name, 'us-west-2')
+
+    def test_new_session_with_unknown_region(self):
+        s3_client = self.session.create_client('s3', 'MyCustomRegion1')
+        self.assertIsInstance(s3_client, client.BaseClient)
+        self.assertEqual(s3_client.meta.region_name, 'MyCustomRegion1')
+
+    def test_new_session_with_invalid_region(self):
+        with self.assertRaises(botocore.exceptions.InvalidRegionError):
+            s3_client = self.session.create_client('s3', 'not.a.real#region')
+
+    def test_new_session_with_none_region(self):
+        s3_client = self.session.create_client('s3', region_name=None)
+        self.assertIsInstance(s3_client, client.BaseClient)
+        self.assertTrue(s3_client.meta.region_name is not None)
